const metaRegex = /<meta[^<>]*?=(['"].*?['"]|[^<>]*?)*?\/?>/g;
const titleRegex = /<title[^<>]*?>(.*?)<\/title>/g;
const attributesRegex = /(\S*?)=("(.*?)"|'(.*?)'|([^<>\s]*))/g;

/**
  Note:
  1. In server side we will add meta tags and title at last after fitering
  2. In client we will match and replace meta tagString
  3. For now we will not support link and other tags properly, they can be added but we will not check for uniqueness and will not decide placement
**/

function getAttributes(tagString) {
  const attr = {};
  if (!tagString) return attr;
  let match;
  while ((match = attributesRegex.exec(tagString)) !== null) {
    attr[match[1]] = match[3] || match[4] || match[5];
  }

  return attr;
}

export function extractMetaAndTitle(domString) {
  let title;
  const metas = [];
  let match;

  //extract title, only take the last title, remove title from the string
  domString = domString.replace(titleRegex, (titleStr) => {
    title = titleStr;
    return '';
  });

  //extract metas
  domString = domString.replace(metaRegex, (tagString) => {
    metas.push({...getAttributes(tagString), tagString});
    return '';
  });

  return {
    title,
    metas,
    rest: domString
  }
}
